<?php

namespace Drupal\openfed_migrate\Plugin\migrate\source\webform;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\MigrateMessage;
use Drupal\migrate\Row;
use Drupal\node\Entity\Node;
use Drupal\webform\Entity\Webform;
use Drupal\webform_migrate\Plugin\migrate\source\d7\D7Webform;
use Symfony\Component\Yaml\Yaml;

/**
 * Drupal 7 webform source from database.
 *
 * @MigrateSource(
 *   id = "d7_webform",
 *   core = {7},
 *   source_module = "webform",
 *   destination_module = "webform"
 * )
 */
class OpenfedWebform extends D7Webform {

  /**
   * {@inheritdoc}
   */
  public function query() {
    $query = parent::query();

    // Add language and tnid fields in order to assign the correct webforms to
    // node translations.
    $query->fields('n', ['language', 'tnid']);

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    parent::prepareRow($row);

    // Send an alert if email handler doesn't exist.
    if (empty($row->getSourceProperty('handlers'))) {
      $message = new MigrateMessage();
      $message->display(
        $this->t(
          "The webform @form_id doesn't have an email handler. Please create one after the migration.\r\n",
          [
            '@form_id' => 'webform_' . $row->getSourceProperty('nid'),
          ]
        ),
        'alert'
      );
    }

    // We'll override the elements generated by parent prepareRow() because we
    // need to handle some issues.
    $nid = $row->getSourceProperty('nid');
    $webform = $this->buildFormElements($nid);
    $elements = $webform['elements'];

    $row->setSourceProperty('elements', $elements);

    return $row;
  }

  /**
   * {@inheritdoc}
   */
  public function postImport(MigrateImportEvent $event) {
    // This is the same code as on D7Webform.php but we need to improve it to
    // make the fields translateable and save the webform on the right node
    // translation.
    // This is a dirty way of creating the webform field but that's what we
    // have...
    // Add the Webform field to the webform content type
    // if it doesn't already exist.
    $field_storage = FieldStorageConfig::loadByName('node', 'webform');
    if (empty($field_storage)) {
      $field_storage = FieldStorageConfig::create(['entity_type' => 'node', 'field_name' => 'webform', 'type' => 'webform', 'settings' => [], 'cardinality' => 1, 'translatable' => 'true']);
      $field_storage->save();
    }
    $field = FieldConfig::loadByName('node', 'webform', 'webform');
    if (empty($field)) {
      $field = entity_create('field_config', [
        'field_name' => 'webform',
        'bundle' => 'webform',
        'label' => 'Webform',
        'translatable' => 'true',
        'entity_type' => 'node',
      ]);
      $field->save();
      // Assign widget settings for the 'default' form mode.
      $display = entity_get_form_display('node', 'webform', 'default')->getComponent('webform');
      entity_get_form_display('node', 'webform', 'default')
        ->setComponent('webform', [
          'type' => $display['type'],
        ])
        ->save();
      // Assign display settings for the 'default' and 'teaser' view modes.
      $display = entity_get_display('node', 'webform', 'default')->getComponent('webform');
      entity_get_display('node', 'webform', 'default')
        ->setComponent('webform', [
          'label' => $display['label'],
          'type' => $display['type'],
        ])
        ->save();
      // The teaser view mode is created by the Standard profile and therefore
      // might not exist.
      $view_modes = \Drupal::entityManager()->getViewModes('node');
      if (isset($view_modes['teaser'])) {
        $display = entity_get_display('node', 'webform', 'teaser')->getComponent('webform');
        entity_get_display('node', 'webform', 'teaser')
          ->setComponent('webform', [
            'label' => $display['label'],
            'type' => $display['type'],
          ])
          ->save();
      }
    }

    // Attach any Webform created to the relevant webforms if
    // Webform exists and Webform exists and Webform field is empty.
    $webforms = $this->query()->execute();
    foreach ($webforms as $webform) {
      $webform_nid = $webform['nid'];
      $webform_id = 'webform_' . $webform_nid;
      $webform_tnid = $webform['tnid'];
      $webform = Webform::load($webform_id);
      if (!empty($webform)) {
        // Change the webform.
        // Attach any Webform created to the relevant webforms if
        // Webform exists and Webform exists and Webform field is empty.
        $webformSettings = $webform->get('settings');
        $webformSettings['results_disabled'] = 1;
        $webformSettings['page'] = 0;
        $elements = $webform->getElementsDecoded();
        if (isset($webformSettings['form_submit_label']) && !empty($webformSettings['form_submit_label'])) {
          $elements['actions'] = [
            '#type' => 'webform_actions',
            '#title' => 'webform_actions',
            '#submit__label' => $webformSettings['form_submit_label'],
          ];
        }
        $webform->set('third_party_settings', ['honeypot' => ['honeypot' => true]]);
        $webform->set('settings', $webformSettings);
        $webform->setElements($elements);
        $webform->save();

        // Change the attached node.
        if (!empty($webform_tnid) && $webform_tnid > 0) {
          $langcode = $webform->getLangcode();
          $node = Node::load($webform_tnid);
          $node = $node->hasTranslation($langcode) ? $node->getTranslation($langcode) : $node;
        } else {
          $node = Node::load($webform_nid);
        }
        if (!empty($node) && $node->getType() == 'webform') {
          $node->webform->target_id = $webform_id;
          $node->webform->status = 'open';
          $node->save();
        }
      }
    }
  }

  /**
   * Build form elements from webform component table.
   */
  private function buildFormElements($nid) {
    // TODO : Use yaml_emit http://php.net/manual/en/function.yaml-emit.php
    $output = '';

    $query = $this->select('webform_component', 'wc');
    $query->fields('wc', [
      'nid',
      'cid',
      'pid',
      'form_key',
      'name',
      'type',
      'value',
      'extra',
      'required',
      'weight',
    ]);
    $components = $query->condition('nid', $nid)->orderBy('pid')->orderBy('weight')->execute();
    $children = [];
    $parents = [];
    $elements = [];
    $xref = [];

    // Build an array of elements in the correct order for rendering based on
    // pid and weight and a cross reference array to match cid with form_key
    // used by email handler.
    $multiPage = FALSE;
    foreach ($components as $component) {
      // CUSTOM: string lower element key as this is a requirement.
      $xref[$component['cid']] = strtolower($component['form_key']);
      if ($component['type'] == 'pagebreak') {
        // Pagebreak found so we have a multi-page form.
        $multiPage = TRUE;
      }
      $children[$component['pid']][] = $component['cid'];
      $parents[$component['cid']][] = $component['pid'];
      $elements[$component['cid']] = $component;
    }
    // Keeps track of the parents we have to process, the last entry is used
    // for the next processing step.
    $process_parents = [];
    $process_parents[] = 0;
    $elements_tree = [];
    // Loops over the parent components and adds its children to the tree array.
    // Uses a loop instead of a recursion, because it's more efficient.
    while (count($process_parents)) {
      $parent = array_pop($process_parents);
      // The number of parents determines the current depth.
      $depth = count($process_parents);
      if (!empty($children[$parent])) {
        $has_children = FALSE;
        $child = current($children[$parent]);
        do {
          if (empty($child)) {
            break;
          }
          $element = &$elements[$child];
          $element['depth'] = $depth;
          // We might get element with same form_key
          // d8 doesn't like that so rename it.
          if ($depth > 0) {
            // CUSTOM: string lower element key as this is a requirement.
            $element['form_key'] = strtolower($element['form_key']) . '_' . $element['pid'];
          }
          unset($element['pid']);
          $elements_tree[] = $element;
          if (!empty($children[$element['cid']])) {
            $has_children = TRUE;
            // We have to continue with this parent later.
            $process_parents[] = $parent;
            // Use the current component as parent for the next iteration.
            $process_parents[] = $element['cid'];
            // Reset pointers for child lists because we step in there more often
            // with multi parents.
            reset($children[$element['cid']]);
            // Move pointer so that we get the correct term the next time.
            next($children[$parent]);
            break;
          }
        } while ($child = next($children[$parent]));

        if (!$has_children) {
          // We processed all components in this hierarchy-level.
          reset($children[$parent]);
        }
      }
    }

    // If form has multiple pages then start first page automatically.
    if ($multiPage) {
      $pageCnt = 1;
      $current_page = 'wizard_page_1';
      $output .= "first_page:\n  '#type': wizard_page\n  '#title': {" . $current_page . "_title}\n  '#open': true\n";
      $current_page_title = t('Page') . ' ' . $pageCnt++;
    }

    foreach ($elements_tree as $element) {
      // Rename fieldsets to it's own unique key.
      if ($element['type'] == 'fieldset' && strpos($element['form_key'], 'fieldset') === FALSE) {
        $element['form_key'] = 'fieldset_' . strtolower($element['form_key']);
      }

      // If this is a multi-page form then indent all elements one level
      // to allow for page elements.
      if ($multiPage && $element['type'] != 'pagebreak') {
        $element['depth'] += 1;
      }
      $indent = str_repeat(' ', $element['depth'] * 2);
      $extra = unserialize($element['extra']);
      $description = $this->cleanString($extra['description']);

      // Create an option list if there are items for this element.
      $options = '';
      $valid_options = [];
      if (!empty($extra['items'])) {
        $items = explode("\n", trim($extra['items']));
        $ingroup = '';
        foreach ($items as $item) {
          $item = trim($item);
          if (!empty($item)) {
            if (preg_match('/^<(.*)>$/', $item, $matches)) {
              // Handle option groups.
              $options .= "$indent    '" . $matches[1] . "':\n";
              $ingroup = str_repeat(' ', 2);
            }
            else {
              $option = explode('|', $item);
              $valid_options[] = $option[0];
              // CUSTOM: Change option string replacement by a valid char.
              if (count($option) == 2) {
                $options .= "$indent$ingroup    '" . str_replace('\'', '’', $option[0]) . "': '" . str_replace('\'', "''", $option[1]) . "'\n";
              }
              else {
                $options .= "$indent$ingroup    '" . str_replace('\'', '’', $option[0]) . "': '" . str_replace('\'', "''", $option[0]) . "'\n";
              }
            }
          }
        }
      }

      // Replace any tokens in the value.
      if (!empty($element['value'])) {
        $element['value'] = $this->replaceTokens($element['value']);
      }

      $markup = $indent . strtolower($element['form_key']) . ":\n";
      switch ($element['type']) {
        case 'fieldset':
          if ($multiPage && empty($current_page_title)) {
            $current_page_title = $element['name'];
          }
          $markup .= "$indent  '#type': fieldset\n$indent  '#open': true\n";
          break;

        case 'textfield':
          $markup .= "$indent  '#type': textfield\n";
          if (!empty($extra['width'])) {
            $markup .= "$indent  '#size': " . $extra['size'] . "\n";
          }
          break;

        case 'textarea':
          $markup .= "$indent  '#type': textarea\n";
          break;

        case 'select':
          if (!empty($extra['aslist'])) {
            $select_type = 'select';
          }
          elseif (!empty($extra['multiple']) && is_array($extra['items']) && count($extra['items']) > 1) {
            $select_type = 'checkboxes';
          }
          elseif (!empty($extra['multiple']) && is_array($extra['items']) && count($extra['items']) == 1
            || !empty($extra['multiple']) && !is_array($extra['items'])) {
            $select_type = 'checkbox';
            $markup .= "$indent  '#type': checkbox\n";
            $option = explode(':', $options);
            unset($option[0]);
            $markup .= "$indent  '#title':" . implode(':', $option) . "\n";
            $markup .= "$indent  '#required': true\n";
          }
          else {
            $select_type = 'radios';
          }

          if ($select_type != 'checkbox') {
            $markup .= "$indent  '#type': $select_type\n";
            $markup .= "$indent  '#options':\n" . $options;
            if (!empty($extra['multiple'])) {
              $markup .= "$indent  '#multiple': true\n";
            }
          }

          break;

        case 'email':
          $markup .= "$indent  '#type': email\n$indent  '#size': 20\n";
          break;

        case 'number':
          if ($extra['type'] == 'textfield') {
            $markup .= "$indent  '#type': textfield\n$indent  '#size': 20\n";
          }
          elseif ($extra['type'] == 'select') {
            $markup .= "$indent  '#type': select\n";
            $markup .= "$indent  '#options':\n" . $options;
            $min = $extra['min'];
            $max = $extra['max'];
            $step = !empty($extra['step']) ? $extra['step'] : 1;
            for ($value = $min; $value <= $max; $value += $step) {
              $markup .= "$indent    " . $value . ": " . $value . "\n";
            }
          }
          else {
            // CUSTOM: set number as the default type for number element.
            $markup .= "$indent  '#type': number\n";
          }
          if (isset($extra['min'])) {
            $markup .= "$indent  '#min': " . $extra['min'] . "\n";
          }
          if (isset($extra['max'])) {
            $markup .= "$indent  '#max': " . $extra['max'] . "\n";
          }
          if (isset($extra['step'])) {
            $markup .= "$indent  '#step': " . $extra['step'] . "\n";
          }
          if (isset($extra['unique'])) {
            $unique = ($extra['unique']) ? 'true' : 'false';
            $markup .= "$indent  '#unique': " . $unique . "\n";
          }
          break;

        case 'markup':
          $markup .= "$indent  '#type': processed_text\n$indent  '#format': flexible_html\n$indent  '#text': \"" . $this->cleanString($element['value']) . "\"\n";
          $element['value'] = '';
          break;

        case 'file':
          $exts = '';
          if (!empty($extra['filtering']['types'])) {
            $types = $extra['filtering']['types'];
            if (!empty($extra['filtering']['addextensions'])) {
              $add_types = explode(',', $extra['filtering']['addextensions']);
              $types = array_unique(array_merge($types, array_map('trim', $add_types)));
            }
            $exts = implode(',', $types);
          }
          $filesize = '';
          if (!empty($extra['filtering']['size'])) {
            $filesize = $extra['filtering']['size'] / 1000;
          }
          $markup .= "$indent  '#type': managed_file\n";
          $markup .= "$indent  '#max_filesize': '$filesize'\n";
          $markup .= "$indent  '#file_extensions': '$exts'\n";
          if (!empty($extra['width'])) {
            $markup .= "$indent  '#size': " . $extra['width'] . "\n";
          }
          break;

        case 'date':
          $markup .= "$indent  '#type': date\n";
          /*if (!empty($element['value'])) {
          $element['value'] = date('Y-m-d', strtotime($element['value']));
          }*/
          break;

        case 'time':
          $markup .= "$indent  '#type': time\n";
          if (!empty($extra['hourformat'])) {
            if ($extra['hourformat'] == '12-hour') {
              $markup .= "$indent  '#time_format': 'g:i A'\n";
            }
            elseif ($extra['hourformat'] == '24-hour') {
              $markup .= "$indent  '#time_format': 'H:i'\n";
            }
          }
          /*if (!empty($element['value'])) {
          $element['value'] = date('c', strtotime($element['value']));
          }*/
          break;

        case 'hidden':
          $markup .= "$indent  '#type': hidden\n";
          break;

        case 'pagebreak':
          $output = str_replace('{' . $current_page . '_title}', $current_page_title, $output);
          $current_page = strtolower($element['form_key']);
          $markup .= "$indent  '#type': wizard_page\n  '#open': true\n  '#title': {" . $current_page . "_title}\n";
          $current_page_title = t('Page') . ' ' . $pageCnt++;
          break;
      }

      // Add common fields.
      if (!empty(trim($element['value'])) && (empty($valid_options) || in_array($element['value'], $valid_options))) {
        $markup .= "$indent  '#default_value': '" . str_replace(array('\'', "\n", "\r"), array('"', '\n', ''), trim($element['value'])) . "' \n";
      }
      if (!empty($extra['field_prefix'])) {
        $markup .= "$indent  '#field_prefix': " . $extra['field_prefix'] . "\n";
      }
      if (!empty($extra['field_suffix'])) {
        $markup .= "$indent  '#field_suffix': " . $extra['field_suffix'] . "\n";
      }
      if (!empty($extra['title_display']) && $extra['title_display'] != 'before') {
        $title_display = $extra['title_display'];
        if ($title_display == 'none') {
          $title_display = 'invisible';
        }
        $markup .= "$indent  '#title_display': " . $title_display . "\n";
      }
      if ($element['type'] != 'pagebreak') {
        $markup .= "$indent  '#title': '" . str_replace('\'', '"', $element['name']) . "' \n";
        $markup .= "$indent  '#description': \"" . $description . "\"\n";
      }
      if (!empty($element['required'])) {
        $markup .= "$indent  '#required': true\n";
      }

      // Build contionals.
      if ($states = $this->buildConditionals($element, $elements)) {
        foreach ($states as $key => $values) {
          $markup .= "$indent  '#states':\n";
          $markup .= "$indent    $key:\n";
          foreach ($values as $value) {
            foreach ($value as $name => $item) {
              $markup .= "$indent      " . Yaml::dump($name, 2, 2) . ":\n";
              $markup .= "$indent        " . Yaml::dump($item, 2, 2);
            }
          }
        }
      }

      $output .= $markup;
    }

    if ($multiPage) {
      // Replace the final page title.
      $output = str_replace('{' . $current_page . '_title}', $current_page_title, $output);
    }
    return ['elements' => $output, 'xref' => $xref];
  }

  /**
   * Translate webform tokens into regular tokens.
   *
   * %uid - The user id (unsafe)
   * %username - The name of the user if logged in.
   *                       Blank for anonymous users. (unsafe)
   * %useremail - The e-mail address of the user if logged in.
   *                       Blank for anonymous users. (unsafe)
   * %ip_address - The IP address of the user. (unsafe)
   * %site - The name of the site
   *             (i.e. Northland Pioneer College, Arizona) (safe)
   * %date - The current date, formatted according
   *              to the site settings.(safe)
   * %nid - The node ID. (safe)
   * %title - The node title. (safe)
   * %sid - The Submission id (unsafe)
   * %submission_url - The Submission url (unsafe)
   * %profile[key] - Any user profile field or value, such as %profile[name]
   *                         or %profile[profile_first_name] (unsafe)
   * %get[key] - Tokens may be populated from the URL by creating URLs of
   *                    the form http://example.com/my-form?foo=bar.
   *                    Using the token %get[foo] would print "bar". (safe)
   * %post[key] - Tokens may also be populated from POST values
   *                      that are submitted by forms. (safe)
   * %email[key] (unsafe)
   * %value[key] (unsafe)
   * %email_values (unsafe)
   * %cookie[key] (unsafe)
   * %session[key] (unsafe)
   * %request[key] (unsafe)
   * %server[key] (unsafe)
   *
   * Safe values are available to all users and unsafe values
   * should only be shown to authenticated users.
   */
  private function replaceTokens($str) {
    return $str;
  }

  /**
   * {@inheritdoc}
   */
  private function cleanString($str) {
    return str_replace(['"', "\n", "\r"], ["'", '\n', ''], $str);
  }

  /**
   * Build conditionals and translate them to states api in D8.
   */
  private function buildConditionals($element, $elements) {
    $nid = $element['nid'];
    $cid = $element['cid'];
    $extra = unserialize($element['extra']);
    // Checkboxes : ':input[name="add_more_locations_24[yes]"]':
    $query = $this->select('webform_conditional', 'wc');
    $query->innerJoin('webform_conditional_actions', 'wca', 'wca.nid=wc.nid AND wca.rgid=wc.rgid');
    $query->innerJoin('webform_conditional_rules', 'wcr', 'wcr.nid=wca.nid AND wcr.rgid=wca.rgid');
    $query->fields('wc', [
      'nid',
      'rgid',
      'andor',
      'weight',
    ])
      ->fields('wca', [
        'aid',
        'target_type',
        'target',
        'invert',
        'action',
        'argument',
      ])
      ->fields('wcr', [
        'rid',
        'source_type',
        'source',
        'operator',
        'value',
      ]);
    $conditions = $query->condition('wc.nid', $nid)->condition('wca.target', $cid)->execute();
    $states = [];
    if (!empty($conditions)) {
      foreach ($conditions as $condition) {
        // Element states.
        switch ($condition['action']) {
          case 'show':
            $element_state = $condition['invert'] ? 'invisible' : 'visible';
            break;

          case 'require':
            $element_state = $condition['invert'] ? 'optional' : 'required';
            break;

          case 'set':
            // Nothing found in D8 :(.
            break;
        }
        // Condition states.
        $operator_value = $condition['value'];
        $depedent = $elements[$condition['source']];
        $depedent_extra = unserialize($depedent['extra']);
        switch ($condition['operator']) {
          case 'equal':
            $element_condition = ['value' => $operator_value];
            if ($depedent['type'] == 'select' && !$depedent_extra['aslist']) {
              $element_condition = ['checked' => TRUE];
            }
            break;

          case 'not_equal':
            // There is no handler for this in D8 so we do the reverse.
            $element_state = $condition['invert'] ? 'visible' : 'invisible';
            $element_condition = ['value' => $operator_value];
            // Specially handle the checkboxes, radios.
            if ($depedent['type'] == 'select' && !$depedent_extra['aslist']) {
              $element_condition = ['checked' => TRUE];
            }
            break;

          case 'less_than':
          case 'less_than_equal':
          case 'greater_than':
          case 'greater_than_equal':
            // Nothing in D8 to handle these.
            break;

          case 'empty':
            if ($operator_value == 'checked') {
              $element_condition = ['unchecked' => TRUE];
            }
            else {
              $element_condition = ['empty' => TRUE];
            }
            break;

          case 'not_empty':
            if ($operator_value == 'checked') {
              $element_condition = ['checked' => TRUE];
            }
            else {
              $element_condition = ['filled' => FALSE];
            }
            break;
        }

        if (!$depedent_extra['aslist'] && $depedent_extra['multiple'] && count($depedent_extra['items']) > 1) {
          $depedent['form_key'] = strtolower($depedent['form_key']) . "[$operator_value]";
        }
        elseif (!$depedent_extra['aslist'] && !$depedent_extra['multiple']) {
          $depedent['form_key'] = strtolower($depedent['form_key']) . "[$operator_value]";
        }
        $states[$element_state][] = [':input[name="' . strtolower($depedent['form_key']) . '"]' => $element_condition];
      }
      if (empty($states)) {
        return FALSE;
      }
      return $states;
    }
    else {
      return FALSE;
    }
  }

}
